# packages loading
import os
import sys

import scipy as sp
import pandas as pd
import numpy as np
import string


# pathDir = os.path.dirname(sys.argv[0])  # Get directory where script is running
# sys.path.append( os.path.join(os.getcwd(),'Module') )  # Append parent directory to path for own modules




class Fit_summary:
    """ .
    loading SSBEC Matlab .mat file for Pandas data processing.
    """
    def __init__(self, fitdataset_mat, Summary_keys):

        """ 
        ---------
        definitions/constants/  (to be given in SI units):
        ---------  
        """
        cLum=2.99792458e8; #% m/s ; exactly defined
        h=6.62607015e-34; #% J.s ; exactly defined
        hbar = h/(2*np.pi);
        kB = 1.380649e-23; #%J/K ; exactly defined
        au = 1.66053906660e-27; #% kg
        a0 = 0.0529177210903e-9; #% Bohr radius, in m
        hbarOverAu = hbar/au;
        auOverkB = au / kB;


        m84Sr =  83.913425; #% au
        m86Sr =  85.9092607309; #% au
        m87Sr =  86.9088774970; #% au
        m88Sr =  87.9056122571; #% au
        mSr = m84Sr; #% au
        a8686 = 823 * a0;
        a8484 = 122.7*a0; #% m
        aScat = a8484; #% m
        lambda_blue = 460.862369e-9; #% m
        gamma=2*np.pi*30.2e6; #% s^-1
        detuning=2*np.pi * 0;
        clebsch=1;
        absCross=clebsch*3*lambda_blue**2/(2*np.pi*(1+(detuning/(2*gamma))**2)); #% m^2
        tof=18e-3; #% time of flight, in s
        atomVelocity = 0.088; #% in m.s^-1 , velocity of atoms in the transport guide
        
        pixel_X_cal = 4.48036*1e-3; #
        pixel_Y_cal = 4.48036*1e-3; #
        

        self.data = pd.DataFrame();      # initialize dataframe
#         self.test_dic  = '';
#         self.test_dic  = 'hello world'
        try:
            self.data['Value']     = fitdataset_mat['xdatagood'][0,:].transpose();   # retriving scanned variables from matlab
#             print('try')
        except:
            print("An exception occurred");
            self.data['Value']     = np.zeros(len(fitdataset_mat['x0series'][:,0])-1)
            
            
        # Loading tables generated by Matlab (fit variables)
        for idx, Fit_variable in enumerate(Summary_keys[2:25]):      # Summary.keys()[2:25]   keys are loaded from ResultsSUmmary.csv
            self.data[Fit_variable]            = fitdataset_mat['x0series'][1:,idx];         # [1:,idx] Parameters(1)= DimpleN; Matlab
            self.data[Fit_variable+'_std']     = fitdataset_mat['x0stdseries'][1:,idx];      # [1:,idx] Parameters(1)= DimpleN; Matlab std


            
            
            
            
        # generating tables for derived parameters/ derived std
        # ' TyDimple'
        self.data[' TyDimple']        = (mSr * auOverkB *(self.data[' x0DimpleWy']*1e-3/2**0.5/tof)**2)*1e6;
        self.data[' TyDimple'+'_std'] = (mSr * auOverkB *(self.data[' x0DimpleWy_std']*1e-3/2**0.5/tof)**2)*1e6

        # ' TyReservoir'
        self.data[' TyReservoir']        = (mSr * auOverkB *(self.data[' x0ReservoirWy']*1e-3/2**0.5/tof)**2)*1e6;
        self.data[' TyReservoir'+'_std'] = (mSr * auOverkB *(self.data[' x0ReservoirWy_std']*1e-3/2**0.5/tof)**2)*1e6

        # ' TyCrossover'
        self.data[' TyCrossover']        = (mSr * auOverkB *(self.data[' x0CrossoverWy']*1e-3/2**0.5/tof)**2)*1e6;
        self.data[' TyCrossover'+'_std'] = (mSr * auOverkB *(self.data[' x0CrossoverWy_std']*1e-3/2**0.5/tof)**2)*1e6

        # ' TyTransport'
        self.data[' TyTransport']        = (mSr * auOverkB *(self.data[' x0TransportWy']*1e-3/2**0.5/tof)**2)*1e6;
        self.data[' TyTransport'+'_std'] = (mSr * auOverkB *(self.data[' x0TransportWy_std']*1e-3/2**0.5/tof)**2)*1e6

        # ' NDimple'
        self.data[' NDimple']            = self.data[' x0DimpleN'] * np.pi * self.data[' x0DimpleWy']*self.data[' x0DimpleWy']\
                                            /(pixel_X_cal*pixel_Y_cal);
        
        self.data[' NDimple'+'_std']     = self.data[' NDimple']* np.sqrt(  (  self.data[' x0DimpleN_std']/self.data[' x0DimpleN'])**2 + 
                                                                            (2*self.data[' x0DimpleWy_std']/self.data[' x0DimpleWy'] )**2 )

        # ' NReservoir'
        self.data[' NReservoir']         = self.data[' x0ReservoirN'] * np.pi * self.data[' x0ReservoirWy']*self.data[' x0ReservoirWx']\
                                            /(pixel_X_cal*pixel_Y_cal);
        
        self.data[' NReservoir'+'_std']  = self.data[' NReservoir']* np.sqrt(  (self.data[' x0ReservoirN_std']/self.data[' x0ReservoirN'])**2 + 
                                                                               (self.data[' x0ReservoirWy_std']/self.data[' x0ReservoirWy'] )**2 +
                                                                               (self.data[' x0ReservoirWx_std']/self.data[' x0ReservoirWx'] )**2 
                                                                            )    
        # ' NCrossover'
        self.data[' NCrossover']        = self.data[' x0CrossoverN'] * np.pi * self.data[' x0CrossoverWy']*self.data[' x0CrossoverWx']\
                                        /(pixel_X_cal*pixel_Y_cal);
        self.data[' NCrossover'+'_std']  = self.data[' NCrossover']* np.sqrt(  (self.data[' x0CrossoverN_std']/self.data[' x0CrossoverN'])**2 + 
                                                                               (self.data[' x0CrossoverWy_std']/self.data[' x0CrossoverWy'] )**2 +
                                                                               (self.data[' x0CrossoverWx_std']/self.data[' x0CrossoverWx'] )**2 
                                                                            )    
        # ' PSD'
        self.data[' PSD']        = self.data[' x0DimpleN'] * 330*740*315\
                                        /(kB*self.data[' TyDimple'])**3;
        
        
            
    def report(self):
        """ 
        Utility to display a brief report for the trap.     
        """
#         return self.data
