#! /usr/bin/env python
# -*- coding: utf-8 -*-
# file: Colors.py
#
# Copyright (c) 2016 Ahmed Omran <ahmed.omran@mpq.mpg.de>

"""
Standard colours library for matplotlib figures.
Defines single colours, but also colourmaps.
These can be created from linear segments between
RGB values or by interpolation using the seaborn library.
"""


from matplotlib.colors import LinearSegmentedColormap
import matplotlib.cm as cm
import matplotlib.pylab as plt
import numpy.random as rand
import seaborn.apionly as sns


### Single colours ###
MyRed = '#B72525'
MyGreen = '#228922'
MyBlue = '#4A4AF3'
MyOrange = '#EF9D0A'
MyTurquoise = '#26A5B4'
MyGrey = '#8F8F8F'
MyRed2 = '#F10E09'
MyBlue2 = '#3257FE'
MyLightGrey = '#DFDFDF'
MyLightRed = '#EABDBD'
MyLightBlue = '#ADADFF'
MyLightGreen = '#BDEEBD'
########################


### Linear segmented colour map: Black via red to white ###
cm_dark_dict = {'blue': ((0.0, 0.0, 0.0),
                      (0.5, 0., 0.),
                      (1.0, 1., 1.)),
  
                'green': ((0.0, 0.0, 0.0),
                       (0.5, 0.0,  0.0),
                       (1, 1.0,  1.0)),

                'red': ((0.0, 0.0, 0.0),
                     (0.25, 0.322, 0.322),
                     (0.35, 0.678, 0.678),
                     (0.5, 1.0,  1.0),
                     (1, 1.0,  1.0))}

cm_dark = LinearSegmentedColormap('cmdark', cm_dark_dict, gamma=0.8)


### Cubehelix colourmap with linear greyscale gradient ###
cm_sns = sns.cubehelix_palette(n_colors=100, light=0.95, dark=0.15,
                               start=1, rot= -1.5, as_cmap=True,
                               reverse=True, hue=1)


### Modify existing colourmaps by changing gamma values ###
cm_pubugn = cm.get_cmap('PuBuGn')
cm_pubugn.set_gamma(0.9)

cm_reds = cm.get_cmap('Reds')
cm_reds.set_gamma(0.9)

cm_orrd = cm.get_cmap('OrRd')
cm_orrd.set_gamma(1.2)

cm_blues = cm.get_cmap('Blues')
cm_blues.set_gamma(1.5)


### Create colourmaps by blending from one colour to another ###
cm_plots = sns.blend_palette(('#2D2D55', '#364FA2', '#3781BA', '#5EBA7D', '#9ED8A4', '#D4ED9B', '#F4FAAE', '#FFFFFF'), as_cmap=True)
cm_plots_r = sns.blend_palette(('#2D2D55', '#364FA2', '#3781BA', '#5EBA7D', '#9ED8A4', '#D4ED9B', '#F4FAAE', '#FFFFFF')[::-1], as_cmap=True)

cm_BlueToLightBlue = sns.blend_palette((MyBlue,MyLightBlue),as_cmap=True)
cm_BlueToRed = sns.blend_palette((MyBlue2,MyRed2),as_cmap=True)
cm_TurquoiseToRed = sns.blend_palette((MyTurquoise,MyRed2),as_cmap=True)
cm_LightBlueToRed = sns.blend_palette((MyLightBlue,MyRed2),as_cmap=True)
cm_LightBlueToOrange = sns.blend_palette((MyLightBlue,MyOrange),as_cmap=True)


if __name__ == '__main__':
    ### Plot image of random data to test colour maps ###
    plt.figure()
    plt.imshow(rand.random((100,100)),cmap = cm_orrd)
    plt.colorbar()
    plt.show()
