#! /usr/bin/env python
# -*- coding: utf-8 -*-
# file: Figures.py
#
# Copyright (c) 2016 Ahmed Omran <ahmed.omran@mpq.mpg.de>

"""
Standard module for creating and formatting matplotlib figures.
"""




# packages loading

import pandas as pd
from pandas import set_option

import scipy as sp
import numpy as np
import os
import csv
import sys
pathDir = os.path.dirname(sys.argv[0])  # Get directory where script is running
sys.path.append( os.path.join(pathDir,'..') )  # Append parent directory to path for own modules
import matplotlib.path as mpath
import matplotlib.patches as mpatches
from matplotlib.patches import PathPatch
from matplotlib.path import Path
#from matplotlib.widgets import Cursor

from PIL import Image


class Data_Vision:
    """ Class which load data from Vision file .
    """
    def __init__(self, path="ini_path", camera_num=1, root_path=''):
        """ Initialise Data class variables.
     
        """
        self.camera           = camera_num;
        self.data_series      = [];   # EX. data_series = ['S33']
        self.date             = [];   # EX. date        = ['180622']
        self.data_legend      = ['legend 1','legend 2'];
        self.rootpath         = [root_path];   # r"Z:\qgases\groups\strontium\SrCAL\Papers\ACStarkDecelerator\Data\reservoir capture velocity range"
    ###########################
    # Vision data information  
    ###########################
        self.CameraNum_data=['DataA.dat', 'DataB.dat', 'DataC.dat', 'DataD.dat', 'DataE.dat', 'DataF.dat', 'DataG.dat']

        
        
    ############################
    #  Building Data series path 
    ############################
    def build_dataset(self):
        filerootpath= [str.replace(self.rootpath[0],"\\","\\")]
        
        
        self.datasets = {} # initialize dictionary of data container for path / legend
        self.data_path = [0]*len(self.data_series)  # initialize dim of datas path container

        if len(self.date) ==1:
            #what will it check? 
            print("date argument is equal to 1")
            for i in range(len(self.data_series)):
                self.data_path[i]=filerootpath[0]+'\\'+self.date[0]+'\\'+ self.data_series[i]+'\\'+self.CameraNum_data[self.camera-1]  # check by print(data_path[i])
                #print(data_series[i])
        else:
            for i in range(len(self.data_series)):
                self.data_path[i]=filerootpath[0]+'\\'+self.date[i]+'\\'+ self.data_series[i]+'\\'+self.CameraNum_data[self.camera-1]  # check by print(data_path[i])
                #print(data_series[i])


        # building data element,  format data{}=["item key": [path,legend]]         
        for i in range(len(self.data_series)):
            self.datasets[self.data_series[i]] = [self.data_path[i], self.data_legend[i]]
   
    def report(self):
        """ Utility to display a brief report for the trap.
        
        """
        print ("Data camera              = {}  " .format(self.camera))
        print ("Data data_series         = {:} " .format(self.data_series))
        print ("Data date                = {:} " .format(self.date))
        print ("Data data_legend         = {:} " .format(self.data_legend))
        print ("Data rootpath            = {:} " .format(self.rootpath))






def targa_reader(filepath, camera):
    d=np.dtype(np.int16).newbyteorder("L")
    rawimport = np.fromfile(filepath,dtype=d)
    if camera == "A": 
        image_array = np.reshape(rawimport[9::],(1004,1002))
    if camera == "B": 
        image_array = np.reshape(rawimport[9::],(1004,1002))
    if camera == "C": 
        image_array = np.reshape(rawimport[9::],(1200,1920))

        
        #    image_array = np.reshape(rawimport[9::],(1002,1004)) # (Andor)        SOMEHOW this dimension works for new images but not EASYBEC pic
#     image_array = np.reshape(rawimport[9::],(1200,1920)) # (PointGrey)  SOMEHOW this dimension works for new images but not EASYBEC pic

    return image_array


def BMP_reader(filepath):
    rawimport_bmp = Image.open(filepath,'r')
    rawimport_bmp = rawimport_bmp.convert('L') #makes it greyscale    
    image_array = np.asarray(rawimport_bmp.getdata(),dtype=np.float64).reshape((rawimport_bmp.size[1],rawimport_bmp.size[0]))
    image_array = np.asarray(image_array,dtype=np.uint8) #if values still in range 0-255! 
    return image_array


def load_image(path_folder_raw, image_num, camera, return_img="OD"):
    path_folder = str.replace(path_folder_raw,"\\","/")

    # The next if-statements have to do with some naming conventions in Vision which I don't quite get yet
    if camera == "A": 
        filename_raw_abs = path_folder+"/P{:04d}AA0.TGA".format(image_num)
        filename_probe = path_folder+"/P{:04d}AP0.TGA".format(image_num)
        filename_noise = path_folder+"/P{:04d}AN0.TGA".format(image_num)
    if camera == "B":
        filename_raw_abs = path_folder+"/P{:04d}BA1.TGA".format(image_num)
        filename_probe = path_folder+"/P{:04d}BP1.TGA".format(image_num)
        filename_noise = path_folder+"/P{:04d}BN1.TGA".format(image_num)
    if camera == "C":
        filename_raw_abs = path_folder+"/P{:04d}CA2.TGA".format(image_num)
        filename_probe = path_folder+"/P{:04d}CP2.TGA".format(image_num)
        filename_noise = path_folder+"/P{:04d}CN2.TGA".format(image_num)        

    data_raw_abs = targa_reader(filename_raw_abs, camera)
    data_probe = targa_reader(filename_probe, camera)
    data_noise = targa_reader(filename_noise, camera)
    
    if return_img == "OD":
        img = -np.log((data_raw_abs-data_noise)/(data_probe-data_noise))
        img[np.isinf(img)] = 0 # Note that for OD, I immediately clean up the images to get rid of nan and infinities 
        img[np.isnan(img)] = 0
        return img
    if return_img == "ODX3":
        img = -np.log((data_raw_abs-data_noise)/(data_probe-data_noise))*3
        img[np.isinf(img)] = 0 # Note that for OD, I immediately clean up the images to get rid of nan and infinities 
        img[np.isnan(img)] = 0
        return img

    
    if return_img == "Absorption":
        return data_raw_abs
    if return_img == "Probe":
        return data_probe

    print("Invalid image requested")
    return None

    
def load_image_bmp(path_folder_raw, image_num, camera): 
    """
    """
    path_folder = str.replace(path_folder_raw,"\\","/")
    if camera == "A": 
        filename_bmp = path_folder+"/P{:04d}AA0.bmp".format(image_num)
    if camera == "B":
        filename_bmp = path_folder+"/P{:04d}BA1.bmp".format(image_num)
    dataImg_bmp = BMP_reader(filename_bmp)
    return dataImg_bmp

def _parse_exp_var(path_folder_raw, image_num, fetch_variable):
    """
    EX:
    _parse_exp_var(path_Beam, 17, 'InitialExpansionTime')
    = 0.1
    
    """
    fetch_var = []
    path_folder = str.replace(path_folder_raw,"\\","/")

    filename_raw = path_folder+"/P{:04d}".format(image_num)+"s.txt"

    
    with open(filename_raw,'r') as data_file:
        csv_data = csv.reader(data_file)
    
        # in case we want to skip lines
#         next(csv_data)
#         next(csv_data)
    
        for idx, line in enumerate(csv_data): 
            if line !=[]:
                string = line[0].split('=')
                if string[0] == fetch_variable:
                    #
                    fetch_var.append(float(string[-1].split('(')[0]))
                    return fetch_var[0]
    return None


def square_path(square_start, width, height):
    """counter-clockwise plotting square, start from bottom left
    """
    return [[square_start[0],square_start[1]],
            [square_start[0]+width,square_start[1]],
            [square_start[0]+width,square_start[1]+height],
            [square_start[0],square_start[1]+height],    
            [square_start[0],square_start[1]]
           ]    

def draw_square_path(x_start, y_start, x_delta, y_delta):
    verts = [
    (y_start, x_start), # left, bottom
    (y_start, x_start + x_delta), # left, top
    (y_start + y_delta , x_start + x_delta ), # left, top
    (y_start + y_delta , x_start), # left, top
    (y_start, x_start), # left, bottom
    ]

    codes = [Path.MOVETO,
             Path.LINETO,
             Path.LINETO,
             Path.LINETO,
             Path.CLOSEPOLY,
             ]
    return Path(verts, codes)   



# path = Path(verts, codes)
